from importlib import import_module
import time
import traceback
import glob
import os
import sys


def autoload_modules():
    modules = []
    modules_path = f"{os.path.dirname(__file__)}/*.py"
    for module_path in glob.glob(modules_path):
        module = os.path.basename(module_path)
        if '__' not in module and '.py' in module:
            modules.append(module.replace('.py', ''))

    return modules


def parse_modules(list_modules=None):
    if list_modules is None:
        list_modules = autoload_modules()
    modules = {}
    for module_name in list_modules:
        retries = 10
        while retries >= 0:
            try:
                if '.py' in module_name:
                    module_path = os.path.dirname(module_name)
                    module_basename = os.path.basename(module_name)
                    module_name = os.path.splitext(module_basename)[0]
                    sys.path.append(module_path)
                    addon = getattr(import_module(f"{module_name}"), 'Addon')
                else:
                    addon = getattr(import_module(f"{__name__}.{module_name}"), 'Addon')
                addon.service = module_name
                modules[module_name] = addon
                retries = -1
                print(f"Loaded addon: {module_name}")
            except ModuleNotFoundError as e:
                print(f"Addon {module_name} is not supported, please remove it from your config")
                print(e)
                retries = -1
            except Exception as e:
                print("----------------")
                print(f"Error with module: {module_name}")
                traceback.print_exc()
                time.sleep(2)
                retries -= 1
    return modules



#!/usr/bin/env python3

import os
import yaml
import time
import json
import distro
import threading
import traceback
import importlib.metadata
import platform
import argparse
import paho.mqtt.client as mqtt
from . import modules
from . import config
from .system_monitor import MonitorSuspend, GracefulKiller

version = importlib.metadata.version(__package__ or __name__)


class LNXlink():

    def __init__(self, config_path):
        print(f"LNXlink {version} started: {platform.python_version()}")

        # Read configuration from yaml file
        self.pref_topic = 'lnxlink'
        self.config = self.read_config(config_path)

        # Run each addon included in the modules folder
        self.Addons = {}
        for service, addon in modules.parse_modules(self.config['modules']).items():
            try:
                tmp_addon = addon(self)
                self.Addons[addon.service] = tmp_addon
            except Exception as e:
                print(f"Error with addon {addon.service}, please remove it from your config")
                traceback.print_exc()

        # Setup MQTT
        self.client = mqtt.Client()
        self.setup_mqtt()

    def publish_monitor_data(self, topic, pub_data):
        # print(topic, pub_data, type(pub_data))
        if pub_data is None:
            return
        if isinstance(pub_data, bool):
            if pub_data is True:
                pub_data = 'ON'
            if pub_data is False:
                pub_data = 'OFF'
        if type(pub_data) == dict:
            if all(v is None for v in pub_data.values()):
                return
        if type(pub_data) == list:
            if all(v is None for v in pub_data):
                return
        if pub_data is None:
            return
        if type(pub_data) in [dict, list]:
            pub_data = json.dumps(pub_data)
        self.client.publish(
            topic,
            payload=pub_data,
            retain=self.config['mqtt']['lwt']['retain']
        )

    def monitor_run(self):
        '''Gets information from each Addon and sends it to MQTT'''
        for service, addon in self.Addons.items():
            if hasattr(addon, 'getInfo') or hasattr(addon, 'getControlInfo'):
                try:
                    subtopic = addon.name.lower().replace(' ', '/')
                    if hasattr(addon, 'getInfo'):
                        topic = f"{self.pref_topic}/{self.config['mqtt']['statsPrefix']}/{subtopic}"
                        pub_data = addon.getInfo()
                        self.publish_monitor_data(topic, pub_data)
                    if hasattr(addon, 'getControlInfo'):
                        topic = f"{self.pref_topic}/monitor_controls/{subtopic}"
                        pub_data = addon.getControlInfo()
                        self.publish_monitor_data(topic, pub_data)
                except Exception as e:
                    print(f"Can't load addon: {service}")
                    traceback.print_exc()

    def monitor_run_thread(self):
        '''Runs method to get sensor information every prespecified interval'''
        self.monitor_run()

        interval = self.config.get('update_interval', 1)
        self.monitor = threading.Timer(interval, self.monitor_run_thread)
        self.monitor.start()

    def setup_mqtt(self):
        '''Creates the mqtt object'''
        self.client = mqtt.Client()
        self.client.on_connect = self.on_connect
        self.client.on_message = self.on_message

        self.client.username_pw_set(self.config['mqtt']['auth']['user'], self.config['mqtt']['auth']['pass'])
        self.client.connect(self.config['mqtt']['server'], self.config['mqtt']['port'], 60)
        self.client.loop_start()

    def read_config(self, config_path):
        '''Reads the config file and prepares module names for import'''
        with open(config_path) as file:
            config = yaml.load(file, Loader=yaml.FullLoader)

        if 'prefix' in config['mqtt'] and 'clientId' in config['mqtt']:
            self.pref_topic = f"{config['mqtt']['prefix']}/{config['mqtt']['clientId']}"
        self.pref_topic = self.pref_topic.lower()

        modules = config.get('modules')
        if modules is not None:
            modules = [x.lower().replace('-', '_') for x in config['modules']]
        config['modules'] = modules
        return config

    def on_connect(self, client, userdata, flags, rc):
        '''Callback for MQTT connect which reports the connection status
        back to MQTT server'''
        print(f"Connected to MQTT with code {rc}")
        client.subscribe(f"{self.pref_topic}/commands/#")
        if self.config['mqtt']['lwt']['enabled']:
            self.client.publish(
                f"{self.pref_topic}/lwt",
                payload=self.config['mqtt']['lwt']['connectMsg'],
                qos=self.config['mqtt']['lwt']['qos'],
                retain=self.config['mqtt']['lwt']['retain']
            )
        if self.config['mqtt']['discovery']['enabled']:
            self.setup_discovery()

    def disconnect(self, *args):
        '''Reports to MQTT server that the service has stopped'''
        print("Disconnected from MQTT.")
        if self.config['mqtt']['lwt']['enabled']:
            self.client.publish(
                f"{self.pref_topic}/lwt",
                payload=self.config['mqtt']['lwt']['disconnectMsg'],
                qos=self.config['mqtt']['lwt']['qos'],
                retain=self.config['mqtt']['lwt']['retain']
            )
        try:
            self.monitor.cancel()
        except Exception as e:
            pass
        self.client.disconnect()

    def temp_connection_callback(self, status):
        '''Report the connection status to MQTT server'''
        if self.config['mqtt']['lwt']['enabled']:
            if status:
                self.client.publish(
                    f"{self.pref_topic}/lwt",
                    payload=self.config['mqtt']['lwt']['disconnectMsg'],
                    qos=self.config['mqtt']['lwt']['qos'],
                    retain=self.config['mqtt']['lwt']['retain']
                )
            else:
                self.client.publish(
                    f"{self.pref_topic}/lwt",
                    payload=self.config['mqtt']['lwt']['connectMsg'],
                    qos=self.config['mqtt']['lwt']['qos'],
                    retain=self.config['mqtt']['lwt']['retain']
                )

    def on_message(self, client, userdata, msg):
        '''MQTT message is received with a module command to excecute'''
        topic = msg.topic.replace(f"{self.pref_topic}/commands/", "")
        message = msg.payload
        print(f"Message received {topic}: {message}")
        try:
            message = json.loads(message)
        except Exception as e:
            message = message.decode()
            # print("String could not be converted to JSON")
            # traceback.print_exc()

        select_service = topic.split('/')
        addon = self.Addons.get(select_service[0])
        if addon is not None:
            if hasattr(addon, 'startControl'):
                try:
                    addon.startControl(select_service, message)
                    self.monitor_run()
                except Exception as e:
                    traceback.print_exc()

    def setup_discovery_monitoring(self, discovery_template, addon, service):
        '''Send discovery information on Home Assistant for sensors'''
        subtopic = addon.name.lower().replace(' ', '/')
        state_topic = f"{self.pref_topic}/{self.config['mqtt']['statsPrefix']}/{subtopic}"

        discovery = discovery_template.copy()
        discovery['name'] = f"{self.config['mqtt']['clientId']} {addon.name}"
        discovery['unique_id'] = f"{self.config['mqtt']['clientId']}_{service}"
        discovery['state_topic'] = state_topic
        discovery['topic'] = state_topic
        if addon.getInfo.__annotations__.get('return') == dict:
            discovery['value_template'] = "{{ value_json.status }}"
            discovery['json_attributes_topic'] = state_topic
            discovery['json_attributes_template'] = "{{ value_json | tojson }}"
        if hasattr(addon, 'icon'):
            discovery['icon'] = addon.icon
        if hasattr(addon, 'unit'):
            discovery['unit_of_measurement'] = addon.unit
        if hasattr(addon, 'title'):
            discovery['title'] = addon.title
        if hasattr(addon, 'entity_picture'):
            discovery['entity_picture'] = addon.entity_picture
        if hasattr(addon, 'device_class'):
            discovery['device_class'] = addon.device_class
        if hasattr(addon, 'state_class'):
            discovery['state_class'] = addon.state_class

        sensor_type = getattr(addon, 'sensor_type', None)
        if sensor_type in ['sensor', 'binary_sensor']:
            discovery['expire_after'] = self.config.get('update_interval', 5) * 2
        if sensor_type is not None:
            self.client.publish(
                f"homeassistant/{sensor_type}/lnxlink/{discovery['unique_id']}/config",
                payload=json.dumps(discovery),
                retain=self.config['mqtt']['lwt']['retain'])

    def setup_discovery_control(self, discovery_template, addon, service, control_name, options):
        '''Send discovery information on Home Assistant for controls'''
        subtopic = addon.name.lower().replace(' ', '/')
        state_topic = f"{self.pref_topic}/monitor_controls/{subtopic}"
        command_topic = f"{self.pref_topic}/commands/{service}/{control_name.replace(' ', '_')}/"
        discovery = discovery_template.copy()
        discovery['name'] = f"{self.config['mqtt']['clientId']} {control_name}"
        discovery['unique_id'] = f"{self.config['mqtt']['clientId']}_{control_name.lower().replace(' ', '_')}"
        discovery['enabled_by_default'] = options.get('enabled', True)
        if 'value_template' in options:
            discovery["value_template"] = options['value_template']
            if options['type'] != 'camera':
                discovery['json_attributes_topic'] = state_topic
                discovery['json_attributes_template'] = "{{ value_json | tojson }}"
        if 'icon' in options:
            discovery['icon'] = options.get('icon', '')
        if 'unit' in options:
            discovery['unit_of_measurement'] = options.get('unit', '')
        if 'title' in options:
            discovery['title'] = options.get('title', '')
        if 'entity_picture' in options:
            discovery['entity_picture'] = options.get('entity_picture', '')
        if 'device_class' in options:
            discovery['device_class'] = options.get('device_class', '')
        if 'state_class' in options:
            discovery['state_class'] = options.get('state_class', '')
        if 'entity_category' in options:
            discovery['entity_category'] = options.get('entity_category', '')

        if options['type'] in ['sensor', 'binary_sensor']:
            discovery['state_topic'] = state_topic
            discovery['expire_after'] = self.config.get('update_interval', 5) * 2
        elif options['type'] in ['camera', 'update']:
            discovery['state_topic'] = state_topic
        elif options['type'] == 'button':
            discovery["command_topic"] = command_topic
            discovery['state_topic'] = f"{self.pref_topic}/lwt"
        elif options['type'] == 'switch':
            discovery["command_topic"] = command_topic
            discovery["state_topic"] = state_topic
            discovery["payload_off"] = "OFF"
            discovery["payload_on"] = "ON"
        elif options['type'] == 'text':
            discovery["command_topic"] = command_topic
            discovery["state_topic"] = state_topic
        elif options['type'] == 'number':
            discovery["command_topic"] = command_topic
            discovery["state_topic"] = state_topic
            discovery["min"] = options.get('min', 1)
            discovery["max"] = options.get('max', 100)
            discovery["step"] = options.get('step', 1)
        elif options['type'] == 'select':
            discovery["command_topic"] = command_topic
            discovery["state_topic"] = state_topic
            discovery["options"] = addon.options
        else:
            print("Not supported:", options['type'])
            return
        self.client.publish(
            f"homeassistant/{options['type']}/lnxlink/{discovery['unique_id']}/config",
            payload=json.dumps(discovery),
            retain=self.config['mqtt']['lwt']['retain'])

    def setup_discovery(self):
        '''First time setup of discovery for Home Assistant'''
        discovery_template = {
            "availability": {
                "topic": f"{self.pref_topic}/lwt",
                "payload_available": "ON",
                "payload_not_available": "OFF",
            },
            "device": {
                "identifiers": [self.config['mqtt']['clientId']],
                "name": self.config['mqtt']['clientId'],
                "model": f"{distro.name()} {distro.version()}",
                "manufacturer": f"LNXlink",
                "sw_version": version,
            },
        }
        for service, addon in self.Addons.items():
            addon = self.Addons[service]
            if hasattr(addon, 'getInfo'):
                try:
                    self.setup_discovery_monitoring(discovery_template, addon, service)
                except Exception as e:
                    traceback.print_exc()
            if hasattr(addon, 'exposedControls'):
                for control_name, options in addon.exposedControls().items():
                    try:
                        self.setup_discovery_control(discovery_template, addon, service, control_name, options)
                    except Exception as e:
                        traceback.print_exc()


def main():
    parser = argparse.ArgumentParser(
        prog="LNXlink",
        description="Send system information to MQTT broker")
    parser.add_argument(
        "-c", "--config",
        help="Configuration file",
        default="/etc/config.yaml",
        required=True)
    args = parser.parse_args()

    config_file = os.path.abspath(args.config)
    config.setup_config(config_file)
    config.setup_systemd(config_file)
    lnxlink = LNXlink(config_file)
    lnxlink.monitor_run_thread()

    # Monitor for system changes (Shutdown/Suspend/Sleep)
    monitor_suspend = MonitorSuspend(lnxlink.temp_connection_callback)
    monitor_suspend.start()
    monitor_gracefulkiller = GracefulKiller(lnxlink.temp_connection_callback)
    while not monitor_gracefulkiller.kill_now:
        time.sleep(0.2)
    monitor_suspend.stop()
    lnxlink.disconnect()


if __name__ == '__main__':
    main()



import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'xdg_open'

    def startControl(self, topic, data):
        print(f"/usr/bin/xdg-open {data}")
        subprocess.call(f"/usr/bin/xdg-open {data}", shell=True)



#!/usr/bin/env python3

import threading
import signal
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import pgi
pgi.install_as_gi()
from gi.repository import GLib


class MonitorSuspend():
    """Monitor DBUS for Suspend and Shutdown events"""

    def __init__(self, callback):
        dbus_loop = DBusGMainLoop(set_as_default=True)  # integrate into main loob
        bus = dbus.SystemBus(mainloop=dbus_loop)        # connect to dbus system wide
        bus.add_signal_receiver(                        # defince the signal to listen to
            callback,                    # name of callback function
            'PrepareForSleep',                          # singal name
            'org.freedesktop.login1.Manager',           # interface
            'org.freedesktop.login1'                    # bus name
        )
        bus.add_signal_receiver(                        # defince the signal to listen to
            callback,                    # name of callback function
            'PrepareForShutdown',                          # singal name
            'org.freedesktop.login1.Manager',           # interface
            'org.freedesktop.login1'                    # bus name
        )
        self.loop = GLib.MainLoop()
        self.t1 = threading.Thread(target=self.loop.run)

    def start(self):
        self.t1.start()

    def stop(self):
        self.loop.quit()
        self.t1.join()


class GracefulKiller():
    """Monitor Signal Termination"""

    def __init__(self, callback):
        self.kill_now = False
        self.callback = callback
        signal.signal(signal.SIGINT, self.exit_gracefully)
        signal.signal(signal.SIGTERM, self.exit_gracefully)

    def exit_gracefully(self, signum, frame):
        print("stopped gracefully")
        self.callback(1)
        self.kill_now = True



from dbus_idle import IdleMonitor


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Idle'
        self.sensor_type = 'sensor'
        self.icon = 'mdi:timer-sand'
        self.unit = 's'
        self.state_class = 'total_increasing'
        self.device_class = 'duration'

    def getInfo(self):
        monitor = IdleMonitor.get_monitor()
        idle_ms = monitor.get_dbus_idle()
        idle_sec = round(idle_ms / 1000, 0)
        return idle_sec



import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Nvidia GPU'
        self.sensor_type = 'sensor'
        self.icon = 'mdi:expansion-card-variant'

    def getInfo(self) -> dict:
        stdout = subprocess.run(
            ['nvidia-smi',
             '--query-gpu=driver_version,name,fan.speed,memory.total,memory.used,memory.free,utilization.gpu,utilization.memory,temperature.gpu,power.draw,clocks.current.graphics',
             '--format=csv,nounits'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8").strip().split('\n')
        # stdout = map(int, stdout.split(','))
        headers = stdout[0].split(', ')
        values = stdout[1].split(', ')

        gpu_status = {}
        for num, header in enumerate(headers):
            gpu_status[header] = values[num]
        gpu_status['status'] = gpu_status['name']
        return gpu_status



import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Suspend'

    def startControl(self, topic, data):
        subprocess.call(["systemctl", "suspend"])

    def exposedControls(self):
        return {
            "suspend": {
                "type": "button",
                "icon": "mdi:progress-clock",
            }
        }



import subprocess
import re


class Addon():

    def __init__(self, lnxlink=None):
        self.name = 'Boot Select'
        self.options = self._get_grub_entries()

    def getControlInfo(self):
        stdout = subprocess.run(
            'grub-editenv list',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8")
        nextentry_pattern = re.compile(r"^next_entry=(\d+)")
        nextentry_match = re.match(nextentry_pattern, stdout)
        entry_ind = 0
        if nextentry_match:
            entry_ind = int(nextentry_match.group(1))

        return self.options[entry_ind]

    def exposedControls(self):
        return {
            "Boot Select": {
                "type": "select",
                "icon": "mdi:harddisk-plus",
                "options": self.options,
            }
        }

    def startControl(self, topic, data):
        ind = self.options.index(data)
        subprocess.call(f"sudo grub-reboot {ind}", timeout=2, shell=True)

    def _get_grub_entries(self):
        menu_pattern = re.compile("^menuentry '([^']*)'")
        submenu_pattern = re.compile("^submenu '([^']*)'")

        grub_entries = []

        with open('/boot/grub/grub.cfg') as f:
            for line in f.readlines():
                menu_entry_match = re.match(menu_pattern, line)
                if menu_entry_match:
                    grub_entry = menu_entry_match.group(1)
                    grub_entries.append(grub_entry)

                submenu_entry_match = re.match(submenu_pattern, line)
                if submenu_entry_match:
                    grub_entry = submenu_entry_match.group(1)
                    grub_entries.append(grub_entry)

        return grub_entries



import psutil


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Disk Usage'
        self.sensor_type = 'sensor'
        self.icon = 'mdi:harddisk'

    def getInfo(self) -> dict:
        disks = {"status": False}
        for disk in psutil.disk_partitions():
            if disk.fstype == 'squashfs':
                continue
            disks[disk.device] = {}
            disk_stats = psutil.disk_usage(disk.mountpoint)
            disks[disk.device]["total"] = self._bytetomb(disk_stats.total)
            disks[disk.device]["used"] = self._bytetomb(disk_stats.used)
            disks[disk.device]["free"] = self._bytetomb(disk_stats.free)
            disks[disk.device]["percent"] = disk_stats.percent
            if disk_stats.percent > 80:
                disks["status"] = True
        return disks

    def _bytetomb(self, byte):
        return round(byte / 1024 / 1024, 1)



import glob
import subprocess
import json
import re


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Microphone used'
        self.icon = 'mdi:microphone'
        self.sensor_type = 'binary_sensor'

        self.use_pactl = subprocess.run(
            f"which pactl && pactl -f json list short source-outputs",
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).returncode == 0


    def getInfo(self):
        if self.use_pactl:
            stdout = subprocess.run(f"pactl -f json list short source-outputs", shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE).stdout.decode("UTF-8")
            # Replace 0,00 values with 0.00
            stdout = re.sub(r'(\s*[+-]?[0-9]+),([0-9]+\s*)',r'\1.\2', stdout)
            data = json.loads(stdout)
            if len(data) > 0:
                return 'ON'
            return "OFF"
        else:
            mics = glob.glob('/proc/asound/**/*c/sub*/status', recursive=True)
            for mic in mics:
                with open(mic) as mic_content:
                    mic_status = mic_content.read().strip().lower()
                    if mic_status != 'closed':
                        return "ON"
            return "OFF"



import cv2


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Webcam'
        self.sensor_type = 'camera'
        self.vid = None

    def getInfo(self):
        if self.vid is not None:
            ret, frame = self.vid.read()
            ret, buffer = cv2.imencode('.jpg', frame)
            frame = buffer.tobytes()
            return frame
        return None

    def getControlInfo(self):
        if self.vid is not None:
            return True
        return False

    def exposedControls(self):
        return {
            "Webcam": {
                "type": "switch",
                "icon": "mdi:webcam",
                "entity_category": "config",
            }
        }

    def startControl(self, topic, data):
        if data.lower() == 'off':
            self.vid.release()
            self.vid = None
        elif data.lower() == 'on':
            self.vid = cv2.VideoCapture(0)



import subprocess
import jc


class Addon():
    def __init__(self, lnxlink):
        self.name = 'Battery'

    def exposedControls(self):
        return {
            "Battery": {
                "type": "sensor",
                "icon": "mdi:battery",
                "unit": "%",
                "device_class": "battery",
                "value_template": "{{ value_json.status }}",
                "enabled": False,
            },
        }


    def getControlInfo(self):
        stdout = subprocess.run(
            'upower --dump',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8")
        upower_json = jc.parse('upower', stdout)

        devices = {'status': None}
        for device in upower_json:
            if 'detail' in device:
                if 'percentage' in device['detail']:
                    name = ''.join([
                        device.get('vendor', ''),
                        device.get('model', ''),
                    ]).strip()
                    if name != "":
                        devices[name] = device['detail']['percentage']
                        if devices['status'] is None:
                            devices['status'] = device['detail']['percentage']
        return devices



import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Restart'
        self.lnxlink = lnxlink

    def startControl(self, topic, data):
        self.lnxlink.temp_connection_callback(True)
        subprocess.call(["shutdown", "-r", "now", "&"])

    def exposedControls(self):
        return {
            "restart": {
                "type": "button",
                "icon": "mdi:restart",
            }
        }



import notify2
import requests
from dbus.mainloop.glib import DBusGMainLoop


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Notify OSD'
        DBusGMainLoop(set_as_default=True)

    def startControl(self, topic, data):
        iconUrl = data.get('iconUrl', '')
        icon = 'notification-message-im'
        if 'http' in iconUrl:
            iconExt = iconUrl.split('.')[-1]
            icon = f"/tmp/lnxlink_notification.{iconExt}"
            img_data = requests.get(iconUrl).content
            with open(icon, 'wb') as handler:
                handler.write(img_data)
        elif iconUrl != '':
            icon = iconUrl

        # notify2
        notify2.init("Test")
        notify2.Notification(
            data['title'],
            data['message'],
            icon
        ).show()



import psutil


class Addon():

    def __init__(self, lnxlink):
        self.name = 'CPU Usage'
        self.sensor_type = 'sensor'
        self.icon = 'mdi:speedometer'
        self.unit = '%'
        self.state_class = 'measurement'

    def getInfo(self):
        return psutil.cpu_percent()



import subprocess
import glob


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Camera used'
        self.icon = 'mdi:webcam'
        self.sensor_type = 'binary_sensor'

    def getInfo(self):
        videos = glob.glob('/dev/video*', recursive=True)
        cam_used = False
        for video in videos:
            proc = subprocess.run(
                f"fuser {video}",
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            if proc.returncode == 0:
                cam_used = True

        if cam_used:
            return "ON"
        return "OFF"



import importlib.metadata
import time
import requests
import traceback


class Addon():

    def __init__(self, lnxlink):
        self.name = 'LNXlink'
        self.last_time = 0
        self.update_interval = 86400  # Check for updates every 24 hours
        version = importlib.metadata.version('lnxlink')
        self.message = {
          "installed_version": version,
          "latest_version": version,
          "release_summary": "",
          "release_url": "https://github.com/bkbilly/lnxlink/releases/latest",
        }

    def exposedControls(self):
        return {
            "Update": {
                "type": "update",
                "title": "LNXlink",
                "icon": "mdi:update",
                "device_class": "firmware",
                "entity_category": "diagnostic",
                "entity_picture": "https://github.com/bkbilly/lnxlink/raw/master/logo.png?raw=true",
            },
        }

    def getControlInfo(self):
        cur_time = time.time()
        if cur_time - self.last_time > self.update_interval:
            self._latest_version()
            self.last_time = cur_time

        return self.message

    def _latest_version(self):
        url = "https://api.github.com/repos/bkbilly/lnxlink/releases/latest"
        try:
            resp = requests.get(url=url).json()
            self.message['latest_version'] = resp['tag_name']
            self.message['release_summary'] = resp['body']
            self.message['release_url'] = resp['html_url']
        except:
            traceback.print_exc()



import subprocess
import time


class Addon():

    def __init__(self, lnxlink):
        self.name = 'System Updates'
        self.last_time = 0
        self.update_interval = 7200  # Check for updates every 2 hours

    def exposedControls(self):
        return {
            "System Updates": {
                "type": "binary_sensor",
                "icon": "mdi:package-variant",
                "entity_category": "diagnostic",
            },
        }

    def getControlInfo(self):
        packages = [
            {
                'command': 'apt list --upgradable | wc -l',
                'logic': '> 2',
            },
            {
                'command': 'yum -q updateinfo list updates | wc -l',
                'logic': '> 2',
            },
        ]

        update_available = False
        cur_time = time.time()
        if cur_time - self.last_time > self.update_interval:
            for package in packages:
                proc = subprocess.run(
                    package['command'],
                    shell=True,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE)
                result = proc.stdout.strip().decode()
                update_available = eval(f"{result}{package['logic']}")
                if update_available:
                    return True
        return update_available



import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Keep Alive'
        self.keepalive = 'OFF'

    def exposedControls(self):
        return {
            "Keep Alive": {
                "type": "switch",
                "icon": "mdi:mouse-variant",
            }
        }

    def getControlInfo(self):
        if self.keepalive == 'ON':
            subprocess.run(
                "xdotool key 0x00",
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE).stdout.decode("UTF-8")

        return self.keepalive

    def startControl(self, topic, data):
        if data.lower() == 'off':
            self.keepalive = 'OFF'
        elif data.lower() == 'on':
            self.keepalive = 'ON'



import psutil
from datetime import datetime


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Network'
        self.timeOld = datetime.now()
        self.recvOld = psutil.net_io_counters().bytes_recv
        self.sentOld = psutil.net_io_counters().bytes_sent

    def exposedControls(self):
        return {
            "Network Upload": {
                "type": "sensor",
                "icon": "mdi:access-point-network",
                "unit": "Mbit/s",
                "state_class": "measurement",
                "device_class": "data_rate",
                "value_template": "{{ value_json.upload }}",
            },
            "Network Download": {
                "type": "sensor",
                "icon": "mdi:access-point-network",
                "unit": "Mbit/s",
                "state_class": "measurement",
                "device_class": "data_rate",
                "value_template": "{{ value_json.download }}",
            },
        }

    def getControlInfo(self):
        """ Returns Mbps"""
        timeNew = datetime.now()
        netio = psutil.net_io_counters()
        recvNew = netio.bytes_recv
        sentNew = netio.bytes_sent

        timeDiff = (timeNew - self.timeOld).total_seconds()
        self.timeOld = timeNew

        recvDiff = recvNew - self.recvOld
        sentDiff = sentNew - self.sentOld
        self.recvOld = recvNew
        self.sentOld = sentNew

        recvSpeed = round(recvDiff * 8 / timeDiff / 1024 / 1024, 2)
        sentSpeed = round(sentDiff * 8 / timeDiff / 1024 / 1024, 2)

        return {
            "upload": recvSpeed,
            "download": sentSpeed,
        }



service_headless = """[Unit]
Description=LNXlink
After=network-online.target

[Service]
Type=simple
Restart=always
User=root

ExecStart={exec_cmd}

[Install]
WantedBy=default.target
"""

service_user = """[Unit]
Description=LNXlink
After=network-online.target multi-user.target graphical.target
PartOf=graphical-session.target
 
[Service]
Type=simple
Restart=always

ExecStart={exec_cmd}
 
[Install]
WantedBy=default.target
"""

config_temp = """
mqtt:
  prefix: 'lnxlink'
  clientId: 'DESKTOP-Linux'
  statsPrefix: 'monitor/stats'
  server: '192.168.1.1'
  port: 1883
  auth:
    user: 'user'
    pass: 'pass'
  discovery:
    enabled: true
  lwt:
    enabled: true
    qos: 1
    retain: true
    connectMsg: 'ON'
    disconnectMsg: 'OFF'
update_interval: 5
modules:
"""



import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'bash'

    def exposedControls(self):
        return {
            "Bash_Command": {
                "type": "text",
                "icon": "mdi:bash",
            }
        }

    def startControl(self, topic, data):
        subprocess.call(f"{data}", shell=True)



from mss import mss
import numpy as np
import cv2


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Screenshot'
        self.sensor_type = 'camera'
        self.sct = None

    def getInfo(self):
        if self.sct is not None:
            sct_img = self.sct.grab(self.sct.monitors[1])
            frame = np.array(sct_img)
            ret, buffer = cv2.imencode('.jpg', frame)
            frame = buffer.tobytes()
            return frame
        return None

    def getControlInfo(self):
        if self.sct is not None:
            return True
        return False

    def exposedControls(self):
        return {
            "Screenshot": {
                "type": "switch",
                "icon": "mdi:monitor-screenshot",
                "entity_category": "config",
            }
        }

    def startControl(self, topic, data):
        if data.lower() == 'off':
            self.sct = None
        elif data.lower() == 'on':
            self.sct = mss()



import psutil


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Memory Usage'

    def exposedControls(self):
        return {
            "Memory Usage": {
                "type": "sensor",
                "icon": "mdi:memory",
                "unit": "%",
                "state_class": "measurement",
                "value_template": "{{ value_json.percent }}",
            },
            "Memory Used": {
                "type": "sensor",
                "icon": "mdi:memory",
                "unit": "MB",
                "state_class": "measurement",
                "device_class": "data_size",
                "value_template": "{{ value_json.used }}",
                "enabled": False,
            },
            "Memory Available": {
                "type": "sensor",
                "icon": "mdi:memory",
                "unit": "MB",
                "state_class": "measurement",
                "device_class": "data_size",
                "value_template": "{{ value_json.available }}",
                "enabled": False,
            },
        }

    def getControlInfo(self):
        vmem = psutil.virtual_memory()
        return {
            "percent": vmem.percent,
            "used": round(vmem.used / 1024 ** 2, 0),
            "available": round(vmem.available / 1024 ** 2, 0),
        }



from dbus.mainloop.glib import DBusGMainLoop
from mpris2 import get_players_uri
from mpris2 import Player
import alsaaudio
import subprocess


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Media Info'
        self.sensor_type = 'sensor'
        self.icon = 'mdi:music'
        self.players = []

    def exposedControls(self):
        return {
            "playpause": {
                "type": "button",
                "icon": "mdi:play-pause",
                "enabled": False,
            },
            "previous": {
                "type": "button",
                "icon": "mdi:skip-previous",
                "enabled": False,
            },
            "next": {
                "type": "button",
                "icon": "mdi:skip-next",
                "enabled": False,
            },
            "volume_set": {
                "type": "number",
                "icon": "mdi:volume-high",
                "min": 0,
                "max": 100,
                "enabled": False,
                "value_template": "{{ value_json.volume }}",
            }
        }

    def startControl(self, topic, data):
        if topic[1] == 'volume_set':
            mixer = alsaaudio.Mixer()
            if data < 1:
                data *= 100
            if data > 100:
                data = 100
            mixer.setvolume(int(data))
        elif topic[1] == 'playpause':
            if len(self.players) > 0:
                self.players[0]['player'].PlayPause()
        elif topic[1] == 'previous':
            if len(self.players) > 0:
                self.players[0]['player'].Previous()
        elif topic[1] == 'next':
            if len(self.players) > 0:
                self.players[0]['player'].Next()
        elif topic[1] == 'play_media':
            url = data["media_id"]
            subprocess.call(["cvlc", "--play-and-exit", url])

    def getInfo(self) -> dict:
        self.__getPlayers()
        info = {
            'title': '',
            'artist': '',
            'album': '',
            'status': 'idle',
            'volume': self.__getVolume(),
            'playing': False
        }
        if len(self.players) > 0:
            player = self.players[0]
            info['playing'] = True
            info['title'] = player['title']
            info['album'] = player['album']
            info['artist'] = player['artist']
            info['status'] = player['status']

        return info

    def __getVolume(self):
        mixer = alsaaudio.Mixer()
        volume = mixer.getvolume()[0]
        if mixer.getmute()[0] == 1:
            volume = 0
        return volume

    def __getPlayers(self):
        DBusGMainLoop(set_as_default=True)
        self.players = []
        for uri in get_players_uri():
            player = Player(dbus_interface_info={'dbus_uri': uri})
            p_status = player.PlaybackStatus
            title = player.Metadata.get('xesam:title')
            artist = player.Metadata.get('xesam:artist')
            album = player.Metadata.get('xesam:album')
            if p_status != 'Stopped':
                artist_str = ''
                if artist is not None:
                    artist_str = ','.join(artist)
                self.players.append({
                    'status': p_status.lower(),
                    'title': str(title),
                    'artist': artist_str,
                    'album': '' if album is None else str(album),
                    'player': player
                })
        return self.players



import os


class Addon():

    def __init__(self, lnxlink):
        self.name = 'Required Restart'

    def exposedControls(self):
        return {
            "Required Restart": {
                "type": "binary_sensor",
                "icon": "mdi:alert-octagon-outline",
                "entity_category": "diagnostic",
            },
        }

    def getControlInfo(self):
        if os.path.exists('/var/run/reboot-required'):
            return "ON"
        else:
            return "OFF"



import subprocess
import re


class Addon():

    def __init__(self, lnxlink=None):
        self.name = 'Brightness'

    def getControlInfo(self):
        displays = self._get_displays()
        avg_brightness = sum(displays.values()) / len(displays.values())

        info = {"status": avg_brightness}
        for display, brightness in displays.items():
            display = display.replace("-", "_")
            info[display] = brightness
        return info

    def exposedControls(self):
        controls = {
            "Brightness": {
                "type": "number",
                "icon": "mdi:brightness-7",
                "min": 0.1,
                "max": 1,
                "step": 0.1,
                "value_template": "{{ value_json.status }}",
            }
        }
        try:
            for display in self._get_displays().keys():
                json_display = display.replace("-", "_")
                controls[f"Brightness {display}"] = {
                    "type": "number",
                    "icon": "mdi:brightness-7",
                    "min": 0.1,
                    "max": 1,
                    "step": 0.1,
                    "value_template": "{{ value_json." + json_display + " }}",
                    "enabled": False,
                }
        except Exception as e:
            print(e)
        return controls

    def startControl(self, topic, data):
        if topic[1] == 'Brightness':
            for display in self._get_displays().keys():
                subprocess.call(f"xrandr --output {display} --brightness {data}", shell=True)
        else:
            display = topic[1].replace('Brightness_', '')
            subprocess.call(f"xrandr --output {display} --brightness {data}", shell=True)

    def _get_displays(self):
        stdout = subprocess.run(
            'xrandr --verbose --current',
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE).stdout.decode("UTF-8")
        pattern = re.compile(r"(\S+) \bconnected\b.*[\s\S]*?(?=Brightness)Brightness: ([\d\.\d]+)")

        displays = {}
        for match in pattern.findall(stdout):
            displays[match[0]] = float(match[1])

        return displays
